from Products.ERP5Type.tests.SecurityTestCase import SecurityTestCase
import time
from StringIO import StringIO
import zipfile
import httplib
import urlparse


class LocalStringIO(StringIO):
  __allow_access_to_unprotected_subobjects__=1
  def __init__(self, filename, *args, **kw):
    self.filename = filename
    StringIO.__init__(self, *args, **kw)


class TestOfficeJSScenario(SecurityTestCase):
  def getTitle(self):
    return "Test OfficeJS Scenario"

  def getBusinessTemplateList(self):
    """
    Tuple of Business Templates we need to install
    """
    return ('officejs_meta',)

  def createNewUser(self,
                 first_name="John_test",
                 last_name="Smith_test",
                 reference="TEST_OFFICEJS_",
                 email="john.smith@test.com"):
    """
    Add new member using script for the website sign-in
    """
    reference += "%s" % time.time()
    user = self.portal.person_module.newContent(
      portal_type="Person",
      reference=reference,
      user_id=reference,
      first_name=first_name,
      last_name=last_name,
      default_email_coordinate_text=email,
      career_role_list=["author"]
    )
    user.validate()
    assignment = user.newContent(portal_type="Assignment", role="author")
    assignment.open()
    return user

  def test_01_user_can_contribute(self):
    timestamp = "%s" % time.time()
    ###################################
    ### Manager: Create user
    person = self.createNewUser()
    self.tic()

    ###################################
    ### User: submit application
    self.logout()
    self.login(person.getUserId())

    import_file = LocalStringIO('your.zip')

    my_zip = zipfile.ZipFile(import_file, 'w')
    my_zip.writestr('index.html', 'My super content')
    my_zip.close()

    version = 'My Bar version %s' % timestamp
    self.portal.ERP5Site_createNewSoftwarePublication(
      import_file,
      'software/application',
      'My Foo App',
      version
    )
    self.tic()

    ###################################
    ### Alarm: extract zip file
    self.logout()
    self.login()
    self.portal.portal_alarms.officejs_process_submit_software_publication\
                             .activeSense()
    self.tic()
    web_section = self.portal.portal_catalog.getResultValue(
      portal_type='Static Web Section',
      title=version
    )

    ###################################
    ### Anonymous: access the new application
    self.logout()
    api_scheme, api_netloc, api_path, api_query, \
      api_fragment = urlparse.urlsplit(self.portal.absolute_url())

    connection = httplib.HTTPConnection(api_netloc)
    connection.request(
      method='GET',
      url='%s/' % web_section.absolute_url()
    )
    response = connection.getresponse()

    self.assertEquals(response.read(), 'My super content')
    self.assertEquals(response.status, 200)
